@@ -25,8 +25,8 @@
 import math
 
 import gi
-gi.require_versions({'Dazzle': '1.0', 'Gfm': '0.1'})
-from gi.repository import Dazzle, GObject, Gio, Gfm, Gtk
+gi.require_version("Gfm", "0.1")
+from gi.repository import GObject, Gio, Gfm, Gtk
 from gi._gi import pygobject_new_full
 
 from gnomemusic.coreartist import CoreArtist
@@ -65,28 +65,31 @@ class CoreModel(GObject.GObject):
     """
 
     __gsignals__ = {
-        "activate-playlist": (
-            GObject.SignalFlags.RUN_FIRST, None, (Playlist,)),
         "artists-loaded": (GObject.SignalFlags.RUN_FIRST, None, ()),
-        "playlist-loaded": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        "playlist-loaded": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         "playlists-loaded": (GObject.SignalFlags.RUN_FIRST, None, ()),
     }
 
+    active_playlist = GObject.Property(type=Playlist, default=None)
     grilo = GObject.Property(type=CoreGrilo, default=None)
     songs_available = GObject.Property(type=bool, default=False)
 
-    def __init__(self, coreselection):
+    def __init__(self, application):
+        """Initiate the CoreModel object
+
+        :param Application application: The Application instance to use
+        """
         super().__init__()
 
         self._flatten_model = None
+        self._player_signal_id = None
+        self._current_playlist_model = None
         self._previous_playlist_model = None
-        self._search_signal_id = None
-        self._song_signal_id = None
 
         self._model = Gio.ListStore.new(CoreSong)
         self._songliststore = SongListStore(self._model)
 
-        self._coreselection = coreselection
+        self._coreselection = application.props.coreselection
         self._album_model = Gio.ListStore()
         self._album_model_sort = Gfm.SortListModel.new(self._album_model)
         self._album_model_sort.set_sort_func(
@@ -104,14 +107,14 @@ class CoreModel(GObject.GObject):
         self._song_search_flatten = Gfm.FlattenListModel.new(CoreSong)
         self._song_search_flatten.set_model(self._song_search_proxy)
 
-        self._album_search_model = Dazzle.ListModelFilter.new(
+        self._album_search_model = Gfm.FilterListModel.new(
             self._album_model)
         self._album_search_model.set_filter_func(lambda a: False)
 
         self._album_search_filter = Gfm.FilterListModel.new(
             self._album_search_model)
 
-        self._artist_search_model = Dazzle.ListModelFilter.new(
+        self._artist_search_model = Gfm.FilterListModel.new(
             self._artist_model)
         self._artist_search_model.set_filter_func(lambda a: False)
 
@@ -119,14 +122,21 @@ class CoreModel(GObject.GObject):
             self._artist_search_model)
 
         self._playlists_model = Gio.ListStore.new(Playlist)
-        self._playlists_model_filter = Dazzle.ListModelFilter.new(
+        self._playlists_model_filter = Gfm.FilterListModel.new(
             self._playlists_model)
         self._playlists_model_sort = Gfm.SortListModel.new(
             self._playlists_model_filter)
         self._playlists_model_sort.set_sort_func(
             self._wrap_list_store_sort_func(self._playlists_sort))
 
-        self.props.grilo = CoreGrilo(self, self._coreselection)
+        self._user_playlists_model_filter = Gfm.FilterListModel.new(
+            self._playlists_model)
+        self._user_playlists_model_sort = Gfm.SortListModel.new(
+            self._user_playlists_model_filter)
+        self._user_playlists_model_sort.set_sort_func(
+            self._wrap_list_store_sort_func(self._playlists_sort))
+
+        self.props.grilo = CoreGrilo(self, application)
         # FIXME: Not all instances of internal _grilo use have been
         # fixed.
         self._grilo = self.props.grilo
@@ -198,7 +208,7 @@ class CoreModel(GObject.GObject):
         return disc_model_sort
 
     def get_artist_album_model(self, media):
-        albums_model_filter = Dazzle.ListModelFilter.new(self._album_model)
+        albums_model_filter = Gfm.FilterListModel.new(self._album_model)
         albums_model_filter.set_filter_func(lambda a: False)
 
         albums_model_sort = Gfm.SortListModel.new(albums_model_filter)
@@ -219,7 +229,7 @@ class CoreModel(GObject.GObject):
                 if song.props.state == SongWidget.State.PLAYING:
                     song.props.state = SongWidget.State.PLAYED
 
-            self.emit("playlist-loaded")
+            self.emit("playlist-loaded", playlist_type)
             return
 
         def _on_items_changed(model, position, removed, added):
@@ -245,6 +255,20 @@ class CoreModel(GObject.GObject):
                         | GObject.BindingFlags.SYNC_CREATE)
 
         with model.freeze_notify():
+            played_states = [SongWidget.State.PLAYING, SongWidget.State.PLAYED]
+            for song in self._playlist_model:
+                if song.props.state in played_states:
+                    song.props.state = SongWidget.State.UNPLAYED
+
+            if self._player_signal_id is not None:
+                self._current_playlist_model.disconnect(self._player_signal_id)
+                self._player_signal_id = None
+                self._current_playlist_model = None
+
+            if (playlist_type != PlayerPlaylist.Type.PLAYLIST
+                    and self.props.active_playlist is not None):
+                self.props.active_playlist = None
+
             self._playlist_model.remove_all()
 
             if playlist_type == PlayerPlaylist.Type.ALBUM:
@@ -255,7 +279,7 @@ class CoreModel(GObject.GObject):
 
                 self._flatten_model = Gfm.FlattenListModel.new(
                     CoreSong, proxy_model)
-                self._flatten_model.connect("items-changed", _on_items_changed)
+                self._current_playlist_model = self._flatten_model
 
                 for model_song in self._flatten_model:
                     song = CoreSong(
@@ -271,7 +295,7 @@ class CoreModel(GObject.GObject):
                         GObject.BindingFlags.BIDIRECTIONAL
                         | GObject.BindingFlags.SYNC_CREATE)
 
-                self.emit("playlist-loaded")
+                self.emit("playlist-loaded", playlist_type)
             elif playlist_type == PlayerPlaylist.Type.ARTIST:
                 proxy_model = Gio.ListStore.new(Gio.ListModel)
 
@@ -281,7 +305,7 @@ class CoreModel(GObject.GObject):
 
                 self._flatten_model = Gfm.FlattenListModel.new(
                     CoreSong, proxy_model)
-                self._flatten_model.connect("items-changed", _on_items_changed)
+                self._current_playlist_model = self._flatten_model
 
                 for model_song in self._flatten_model:
                     song = CoreSong(
@@ -297,11 +321,9 @@ class CoreModel(GObject.GObject):
                         GObject.BindingFlags.BIDIRECTIONAL
                         | GObject.BindingFlags.SYNC_CREATE)
 
-                self.emit("playlist-loaded")
+                self.emit("playlist-loaded", playlist_type)
             elif playlist_type == PlayerPlaylist.Type.SONGS:
-                if self._song_signal_id:
-                    self._songliststore.props.model.disconnect(
-                        self._song_signal_id)
+                self._current_playlist_model = self._songliststore.props.model
 
                 for song in self._songliststore.props.model:
                     self._playlist_model.append(song)
@@ -309,23 +331,17 @@ class CoreModel(GObject.GObject):
                     if song.props.state == SongWidget.State.PLAYING:
                         song.props.state = SongWidget.State.PLAYED
 
-                self._song_signal_id = self._songliststore.props.model.connect(
-                    "items-changed", _on_items_changed)
-
-                self.emit("playlist-loaded")
+                self.emit("playlist-loaded", playlist_type)
             elif playlist_type == PlayerPlaylist.Type.SEARCH_RESULT:
-                if self._search_signal_id:
-                    self._song_search_flatten.disconnect(
-                        self._search_signal_id)
+                self._current_playlist_model = self._song_search_flatten
 
                 for song in self._song_search_flatten:
                     self._playlist_model.append(song)
 
-                self._search_signal_id = self._song_search_flatten.connect(
-                    "items-changed", _on_items_changed)
-
-                self.emit("playlist-loaded")
+                self.emit("playlist-loaded", playlist_type)
             elif playlist_type == PlayerPlaylist.Type.PLAYLIST:
+                self._current_playlist_model = model
+
                 for model_song in model:
                     song = CoreSong(
                         model_song.props.media, self._coreselection,
@@ -341,8 +357,11 @@ class CoreModel(GObject.GObject):
                         GObject.BindingFlags.BIDIRECTIONAL
                         | GObject.BindingFlags.SYNC_CREATE)
 
-                self.emit("playlist-loaded")
+                self.emit("playlist-loaded", playlist_type)
 
+        if self._current_playlist_model is not None:
+            self._player_signal_id = self._current_playlist_model.connect(
+                "items-changed", _on_items_changed)
         self._previous_playlist_model = model
 
     def stage_playlist_deletion(self, playlist):
@@ -368,16 +387,6 @@ class CoreModel(GObject.GObject):
         """
         self.props.grilo.create_playlist(playlist_title, callback)
 
-    def activate_playlist(self, playlist):
-        """Activates a playlist.
-
-        Selects the playlist and start playing.
-
-        :param Playlist playlist: playlist to activate
-        """
-        # FIXME: just a proxy
-        self.emit("activate-playlist", playlist)
-
     def search(self, text):
         self.props.grilo.search(text)
 
@@ -420,7 +429,7 @@ class CoreModel(GObject.GObject):
         return self._playlist_model_sort
 
     @GObject.Property(
-        type=Dazzle.ListModelFilter, default=None,
+        type=Gfm.FilterListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def songs_search(self):
         return self._song_search_flatten
@@ -432,7 +441,7 @@ class CoreModel(GObject.GObject):
         return self._song_search_proxy
 
     @GObject.Property(
-        type=Dazzle.ListModelFilter, default=None,
+        type=Gfm.FilterListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def albums_search(self):
         return self._album_search_model
@@ -444,7 +453,7 @@ class CoreModel(GObject.GObject):
         return self._album_search_filter
 
     @GObject.Property(
-        type=Dazzle.ListModelFilter, default=None,
+        type=Gfm.FilterListModel, default=None,
         flags=GObject.ParamFlags.READABLE)
     def artists_search(self):
         return self._artist_search_model
@@ -476,3 +485,15 @@ class CoreModel(GObject.GObject):
         flags=GObject.ParamFlags.READABLE)
     def playlists_filter(self):
         return self._playlists_model_filter
+
+    @GObject.Property(
+        type=Gfm.SortListModel, default=None,
+        flags=GObject.ParamFlags.READABLE)
+    def user_playlists_sort(self):
+        return self._user_playlists_model_sort
+
+    @GObject.Property(
+        type=Gfm.SortListModel, default=None,
+        flags=GObject.ParamFlags.READABLE)
+    def user_playlists_filter(self):
+        return self._user_playlists_model_filter
